"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const pretty_format_1 = __importDefault(require("pretty-format"));
const timezone_utils_1 = __importDefault(require("timezone-utils"));
const path_1 = __importDefault(require("path"));
const util_1 = __importDefault(require("util"));
const fs_1 = __importDefault(require("fs"));
const chalk_1 = __importDefault(require("chalk"));
var Level;
(function (Level) {
    Level["ERROR"] = "error";
    Level["WARN"] = "warn";
    Level["INFO"] = "info";
    Level["DEBUG"] = "debug";
})(Level || (Level = {}));
const options = {
    level: process.env.LOG_LEVEL || (process.env.DEBUG ? Level.DEBUG : Level.INFO),
    timezone: new timezone_utils_1.default().timezone,
    filePath: undefined,
    timed_logs: true
};
class Logger {
    constructor(opt = {}) {
        this.LEVELS = Level;
        this.options = options;
        /* Non-Static Methods */
        this.log = (level, ...messages) => Logger.log(level, ...messages);
        this.info = (...messages) => Logger.log(Level.INFO, ...messages);
        this.debug = (...messages) => Logger.log(Level.DEBUG, ...messages);
        this.warn = (...messages) => Logger.log(Level.WARN, ...messages);
        this.error = (error, when) => Logger.error(error, when);
        if (Object.values(this.LEVELS).indexOf(opt.level) === -1)
            opt.level = this.LEVELS.INFO;
        if (opt.filePath)
            opt.timed_logs = false;
        Object.assign(options, opt);
    }
    /* Main Method */
    static log(level, ...messages) {
        if (Object.values(this.LEVELS).indexOf(level) > Object.values(this.LEVELS).indexOf(this.options.level))
            return;
        const titles = {};
        titles[this.LEVELS.ERROR] = chalk_1.default.red(this.LEVELS.ERROR.toUpperCase());
        titles[this.LEVELS.WARN] = chalk_1.default.yellow(this.LEVELS.WARN.toUpperCase());
        titles[this.LEVELS.INFO] = chalk_1.default.blue(this.LEVELS.INFO.toUpperCase());
        titles[this.LEVELS.DEBUG] = chalk_1.default.yellowBright(this.LEVELS.DEBUG.toUpperCase());
        let error;
        if (messages[1] instanceof Error && level === this.LEVELS.ERROR) {
            error = messages[1];
            messages = messages.slice(0, 1);
        }
        const formatted = messages.map((m) => {
            if (typeof m === "string")
                return m;
            return pretty_format_1.default(m, {
                highlight: true,
                min: true,
                theme: {
                    boolean: "yellow",
                    bracket: "grey",
                    comma: "grey",
                    content: "reset",
                    date: "green",
                    error: "red",
                    function: "blue",
                    key: "cyan",
                    label: "blue",
                    misc: "grey",
                    number: "green",
                    prop: "yellow",
                    regex: "red",
                    string: "reset",
                    symbol: "red",
                    tag: "cyan",
                    value: "green",
                },
            });
        }).map((m) => m.replace(/^(?!\s+$)/gm, " ".repeat(13)).trim());
        let enabled = this.options.filePath || this.options.timed_logs;
        if (!this.fileWriteStream && enabled) {
            if (this.options.filePath && !this.options.timed_logs) {
                let i = 1;
                while (fs_1.default.existsSync(path_1.default.resolve(this.options.filePath))) {
                    let dir = path_1.default.dirname(this.options.filePath);
                    let fileName = path_1.default.basename(this.options.filePath).split(".").shift();
                    let ext = path_1.default.extname(this.options.filePath);
                    this.options.filePath = `${dir}/${fileName ? fileName.replace(/-\d+/g, "") : ""}-${i}${ext}`;
                    i++;
                }
                this.fileWriteStream = fs_1.default.createWriteStream(path_1.default.resolve(this.options.filePath));
            }
            else {
                let time = new timezone_utils_1.default(this.options.timezone);
                if (!fs_1.default.existsSync(path_1.default.resolve("./logs")))
                    fs_1.default.mkdirSync(path_1.default.resolve("./logs"));
                this.fileWriteStream = fs_1.default.createWriteStream(path_1.default.resolve(`./logs/${time.date()} ${time.format("HH-mm-ss")}.log`));
            }
        }
        process.stdout.write(util_1.default.format.apply(arguments, [chalk_1.default.grey(new timezone_utils_1.default(this.options.timezone).time()), `[${titles[level]}]`, ...formatted]) + "\n");
        if (enabled && this.fileWriteStream)
            this.fileWriteStream.write(`${new timezone_utils_1.default(this.options.timezone).time()} [${level.toUpperCase()}] ${formatted.join(" ")}\n`);
        if (error) {
            process.stdout.write(util_1.default.format.apply(arguments, [error]) + "\n");
            if (enabled && this.fileWriteStream)
                this.fileWriteStream.write(error.stack + "\n");
        }
    }
}
Logger.LEVELS = Level;
Logger.options = options;
/* Static Methods */
Logger.info = (...messages) => Logger.log(Level.INFO, ...messages);
Logger.debug = (...messages) => Logger.log(Level.DEBUG, ...messages);
Logger.warn = (...messages) => Logger.log(Level.WARN, ...messages);
Logger.error = (error, when) => {
    if (typeof error === "string")
        error = new Error(error);
    return Logger.log(Level.ERROR, `An error occurred while ${when ? when : "doing important stuff..."}`, error);
};
exports.Logger = Logger;
